;**************************************************************************
;*
;* FILE  boot.asm
;* Copyright (c) 2005, 2006, 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: boot.asm,v 1.25 2008-02-18 09:29:21 tlr Exp $
;*
;* DESCRIPTION
;*   Fast C64 DTV server.
;*
;******
	processor 6502

;dtv SAC #<imm>
	MAC	sac
	dc.b	$32,{1}
	ENDM

;dtv SIR #<imm>
	MAC	sir
	dc.b	$42,{1}
	ENDM

;dtv BRA <rel>
	MAC	bra
_TMPBRA	set	{1}-2-.
	if	_TMPBRA < -128 || _TMPBRA > 127
	ERROR 	"Branch to long!"
	endif
	dc.b	$12,_TMPBRA
	ENDM

;align with nops to an even 4 byte + offs
	MAC	nopalign
	ds.b	[4+{1}-[. & 3]]&3,$ea
	ENDM
		
DEFAULT_REG8	equ	%01010101	;all ram

	seg	code
	org	$4000
start_code:
;**************************************************************************
;*
;* NAME  reset hook ($018000)
;*
;* DESCRIPTION
;*   warm start is used by the normal mem trampolines.
;*
;******
	dc.w	cold_start
	dc.w	warm_start
	dc.b	"D"|$80,"T"|$80,"V"|$80,"8","0"
	dc.w	dtvtrans_hook_start

	include	"config.asm"

;**************************************************************************
;*
;* NAME  cold_start, warm_start
;*
;* DESCRIPTION
;*   Entry Point.
;*
;******
; cold_start is called with the code executed from $8000, not $4000.
cold_start	equ	.+$4000
; this should already have been done, but just to be safe
	sei
	cld
	ldx	#$ff
	txs

	sir	$ed
; Set $8000-$bfff mapping as $4000-$7fff mapping
	tya	; Acc=reg $e
	tax	; reg $d=Acc
	jmp	cs_continue
; from now on the code is executed at $4000
cs_continue:
	ldy	#$02		; Map $008000 -> $8000
	sir	$12

	ldx	#$00
	stx	$d020
	stx	$d011

	jsr	clone_f951

        jsr     clone_fd50_lite
        jsr     clone_fd15
        jsr     $ff5b

; done, proceed to starting the actual server
warm_start:
	sei
	cld
	sac	$00
	sir	$12
	
	lda	cfg_dtvtrans_port
	eor	#$ff
	jsr	server

	ldx	#trampoline_end-trampoline-1
mn_lp1:
	lda	trampoline,x
	sta	trampoline_run,x
	dex
	bpl	mn_lp1

	sac	$dd
	jmp	trampoline_run

trampoline:
	rorg	$110
trampoline_run:
	lda	#$01		; default mapping
	sac	$00
	cli
	jmp	$a474
	rend
trampoline_end:
	

;**************************************************************************
;*
;* NAME  dtvtrans_hook_start
;*
;******
dtvtrans_hook_start:
	php
	sei
	cld
	sac	$00
	sir	$12

	lda	cfg_dtvtrans_port
	eor	#$ff
	jsr	server

	ldx	#trampoline2_end-trampoline2-1
dhs_lp1:
	lda	trampoline2,x
	sta	trampoline2_run,x
	dex
	bpl	dhs_lp1

	sac	$dd
	jmp	trampoline2_run

trampoline2:
	rorg	$0110
trampoline2_run:
	lda	#$01		;bank 1 ($004000)
	sac	$00		;restore default Acc mapping.
	plp
	rts
	rend
trampoline2_end:

	
;**************************************************************************
;*
;* NAME  preserve_scr, restore_scr
;*
;* DESCRIPTION
;*
;******
preserve_scr:
	ldx	#40-1
ps_lp1:
	lda	$0400,x
	sta	line_store,x
	lda	$d800,x
	sta	col_store,x
	dex
	bpl	ps_lp1
	rts

restore_scr:
	ldx	#40-1
rs_lp1:
	lda	line_store,x
	sta	$0400,x
	lda	col_store,x
	sta	$d800,x
	dex
	bpl	rs_lp1
	rts
		
;**************************************************************************
;*
;* NAME  clone_fd50, clone_fd50_lite
;*
;* DESCRIPTION
;*   Clone the memory init at $fd50.
;*
;******
clone_fd50_lite:
	lda	#0
	sta	$c6
	sta	$c7
	ldx	#14
cfd_lp1:
	sta	$8c,x
	sta	$d0,x
	dex
	bpl	cfd_lp1

	jsr	$fda3
	jsr	$e453	;InitBasic

	ldx	#$c0
	stx	$9d	;Status
	lda	$030f
	and	#$f3
	sta	$030f
	ldy	#0
	sty	$02a1	
	sty	$0281

	bra	cfd50_ex1

clone_fd50:
	lda	#0
	tay
cfd50_lp1:
	sta	$0002,y
	sta	$0200,y
	sta	$0300,y
	iny
	bne	cfd50_lp1

cfd50_ex1:
	ldx	#$03
	lda	#$3c
	sta	$b2
	stx	$b3

	ldx	#$00
	ldy	#$a0
	jmp	$fd8c	;Set MemBounds

;**************************************************************************
;*
;* NAME  clone_fd15
;*
;* DESCRIPTION
;*   Clone the vector setup at $fd15.
;*
;******
clone_fd15:
	ldy	#$1f
cfd15_lp1:
	lda	$fd30,y
	sta	$0314,y
	dey
	bpl	cfd15_lp1
	rts


;**************************************************************************
;*
;* NAME  clone_f951
;*
;* DESCRIPTION
;*   Clone of the DTV video init at $f951.
;*
;******
clone_f951:
	ldx	#$01
	stx	$d03f

; is the config all $ff's?
	ldx	#5
	lda	#$ff
cri_lp1:
	and	cfg_hard_video,x
	dex
	bpl	cri_lp1
	cmp	#$ff
	bne	clone_f951_hard	; no, skip to hard video
; yes, do normal config

	lda	#$00
	sta	$dd03
	lda	$dd01
	sta	$aa
	and	#$03
	sta	$d040
	lda	$aa
	lsr
	lsr
	and	#$0f
	sta	$d04f
	bit	$aa
	bvs	cri_skp1
	lda	#7
	dc.b	$2c
cri_skp1:
	lda	#6
	sta	$d04e
	bit	$aa
	bmi	cri_skp2
	ldy	#$02
	dc.b	$2c
cri_skp2:
	ldy	#$05
	lda	$01
	and	#$10
	bne	cri_skp3
	tya
	clc
	adc	#$06
	tay
	lda	#$00
	sta	$d04e
cri_skp3:
	ldx	#$2
cri_lp2:
	lda	consts,y
	sta	$d041,x
	dey
	dex
	bpl	cri_lp2
cri_ex1:
	ldx	#$00
	stx	$d03f
	rts
consts:
	dc.b	$1c,$13,$2a
	dc.b	$24,$31,$5b
	dc.b	$1c,$00,$00
	dc.b	$24,$00,$00

clone_f951_hard:
	ldx	#3
cfh_lp1:
	lda	cfg_hard_video,x
	sta	$d040,x
	dex
	bpl	cfh_lp1
	ldx	#1
cfh_lp2:
	lda	cfg_hard_video+4,x
	sta	$d04e,x
	dex
	bpl	cfh_lp2
	bra	cri_ex1
	
;**************************************************************************
;*
;* external code
;*
;******
	include	"server.asm"

; set up origin for bss.
end_code:
	seg.u	bss
	org	end_code
;**************************************************************************
;*
;* bss data
;*
;******
line_store:
	ds.b	40
col_store:
	ds.b	40	
end_bss:

TOTAL_SIZE	equ	end_bss-start_code
	echo	"binary range:",start_code,"-",end_code	
	echo	"total range:",start_code,"-",end_bss
	echo	"TOTAL_SIZE:",TOTAL_SIZE
; eof
