;**************************************************************************
;*
;* FILE  server.asm
;* Copyright (c) 2005, 2006, 2007, 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: server.asm,v 1.99 2008-02-17 16:17:25 tlr Exp $
;*
;* DESCRIPTION
;*   Fast C64 DTV server.
;*
;* REQUIREMENTS
;*   - IMPLEMENTATION_STR must be defined to a string describing this
;*     program, e.g "DTVTRANS x.y" or "DTVBOOT x.y"
;*
;*   - preserve_scr, restore_scr must exist.  They shall preserve and restore
;*     the contents of the first line on screen. (best effort)
;*
;*   - clone_fd50, clone_fd15 must exist.
;*   - clone_fd50_lite must also exist.  It shall perform the minimal amount
;*     of fd50 low mem init + $fda3 + init basic vectors $e453.
;*
;*   - TOTAL_SIZE must define where the end of the dtvtrans code is in
;*     relation to the start of the current segment.
;*
;******

; Protocol revision (generally tracks the dtvtrans release where the
; changes were introduced)
PREV_MAJOR	equ	1
PREV_MINOR	equ	0
	
;***
; Protocol definitions

; memory type commands
CMD_READ_MEM	equ	$01
CMD_WRITE_MEM	equ	$02
CMD_EXECUTE_MEM	equ	$03
CMD_SYS		equ	$04
CMD_SYS_RESULT	equ	$05

; basic type commands
CMD_INIT	equ	$40
CMD_LOAD	equ	$41
CMD_RUN		equ	$42
CMD_EXIT	equ	$43

; screen type commands
CMD_PRINT	equ	$60
CMD_COLOR	equ	$61

; configuration type commands
CMD_QUERY_REVISION equ	$80
CMD_QUERY_IMPLEMENTATION equ $81
CMD_QUERY_CONFIG equ	$82
CMD_SET_BLOCKCTRL equ	$83

; special commands
CMD_NOP		equ	$ff

;***
; Speed code definitions
SPEED_NOTINSTALLED	equ	0
SPEED_PUTBLOCK		equ	1
SPEED_PUTBLOCK_SAFE	equ	2
SPEED_GETBLOCK		equ	3


	seg.u	zp_srv
	org	$f9
;**************************************************************************
;*
;* SECTION  zp_srv
;*
;* DESCRIPTION
;*   Zeropage area allocation
;*
;******
zp_srv_start:
mode_zp:
	ds.b	1
bank_zp:
	ds.b	1
ptr_zp:
	ds.w	1
end_zp:
	ds.w	1
chk_zp:
	ds.b	1
zp_srv_end:
	echo	"[dtvtrans] zp usage:",zp_srv_start,"-",zp_srv_end

	seg.u	stack_srv
	org	$0108
;**************************************************************************
;*
;* SECTION  stack_srv
;*
;* DESCRIPTION
;*   Stack area allocation
;*
;******
stack_srv_start:

;*** stack preservation area
STACKCOPY_LEN	equ	8
STACKCOPY_ADDR	equ	$01f8
sp_store:
	ds.b	1
stack_store:
	ds.b	STACKCOPY_LEN

;*** register store area
store_sr:
	ds.b	1
store_acc:
	ds.b	1
store_xr:
	ds.b	1
store_yr:
	ds.b	1
; temporary mode register
mode_tmp:
	ds.b	1
	
;*** jumptable area
	org	$0120
jumptable:	
jumptable_getbyte:
	ds.w	1
jumptable_putbyte:
	ds.w	1
jumptable_getmode:
	ds.w	1
jumptable_putmode:
	ds.w	1
jumptable_checkclk:
	ds.w	1
jumptable_restore:
	ds.w	1
JUMPTABLE_LEN	equ	.-jumptable

;*** misc storage
store_port:
	ds.b	1
current_speedcode:
	ds.b	1
blockctrl_reg:
	ds.b	1
current_progress:
	ds.b	1
store_d020:
	ds.b	1
store_reg_e:
	ds.b	1
store_reg_8:
	ds.b	1

;*** speed code storage
	align	4
block_speedcode:
	ds.b	LARGEST_SPEEDCODE
stack_srv_end:

	echo	"[dtvtrans] stack usage:",stack_srv_start,"-",stack_srv_end
	
	seg	code
;**************************************************************************
;*
;* NAME  server
;*
;* DESCRIPTION
;*   Initialize for communication.
;*
;******
server:
; set up jumptable
	sta	store_port
	asl
	tax
	lda	porttab,x
	sta	ptr_zp
	lda	porttab+1,x
	sta	ptr_zp+1
	ldy	#JUMPTABLE_LEN-1
srv_lp1:
	lda	(ptr_zp),y
	sta	jumptable,y
	dey
	bpl	srv_lp1
; initialize
	jsr	init_scr_comm
srv_mainloop:
	jsr	getmode

; normal wait for command.
	jsr	clear_progress
	jsr	wait_command
	bcs	srv_exit
	
srv_getcommand:
	jsr	getbyte
	ldx	#0
srv_lp2:
	cmp	commands,x
	beq	srv_found
	inx
	inx
	inx
	cpx	#COMMANDS_LEN
	bne	srv_lp2
; invalid command
 	lda	#2		
 	sta	$d020
	bra	srv_return
	
srv_exit:
	jmp	uninit_scr_comm
; 	rts			

	
srv_found:
	lda	#5
	sta	$d020
	lda	#>[srv_return-1]
	pha
	lda	#<[srv_return-1]
	pha
	lda	commands+1,x
	pha
	lda	commands+2,x
	pha
srv_cmd_nop:
	rts
srv_return:
	jsr	getmode
; check for a new command for a short while before reseting
; colors and such so there won't be graphic glitches during
; chained commands
	ldy	#0
srv_lp3:
	jsr	checkclk
	beq	srv_getcommand
	iny
	bne	srv_lp3
	bra	srv_mainloop
	
; command table
	mac	scmd
	dc.b	{1}
	dc.b	>[{2}-1]
	dc.b	<[{2}-1]
	endm
commands:
	scmd	CMD_WRITE_MEM,      srv_cmd_write_mem
	scmd	CMD_READ_MEM,       srv_cmd_read_mem
	scmd	CMD_EXECUTE_MEM,    srv_cmd_execute_mem
	scmd	CMD_SYS,            srv_cmd_sys
	scmd	CMD_SYS_RESULT,     srv_cmd_sys_result
	scmd	CMD_QUERY_REVISION, srv_cmd_query_revision
	scmd	CMD_QUERY_IMPLEMENTATION, srv_cmd_query_implementation
	scmd	CMD_QUERY_CONFIG,   srv_cmd_query_config
	scmd	CMD_SET_BLOCKCTRL,  srv_cmd_set_blockctrl
	scmd	CMD_INIT,           srv_cmd_init
	scmd	CMD_LOAD,           srv_cmd_load
	scmd	CMD_RUN,            srv_cmd_run
	scmd	CMD_EXIT,           srv_cmd_exit
	scmd	CMD_PRINT,          srv_cmd_print
	scmd	CMD_COLOR,          srv_cmd_color
	scmd	CMD_NOP,	    srv_cmd_nop	; just rts
COMMANDS_LEN	equ	.-commands

;**************************************************************************
;*
;* NAME  wait_command
;*
;* DESCRIPTION
;*   Wait for clock to change or Esc to be pressed, and generate border
;*   pattern.  C=0 normal, C=1 Esc pressed.
;*
;******
wait_command:
; run without burst mode to have the same appearance in both flash and ram
; versions.
	sir	$19
	ldx	#%00000001
	sir	$12
; main loop
wc_lp1:
	lda	#0
	sta	$d020
	jsr	time3
	jsr	time3
	jsr	time2
 	ldx	#%11111111
 	lda	#%01111111	
	sta	$dc00
	lda	$dc01
	and	#%10000000	; run/stop
	stx	$dc00
	beq	wc_ex2
	lda	#11
	sta	$d020

	jsr	checkclk
	bne	wc_lp1
	clc
wc_ex1:
; restore burst mode
	sir	$19
	ldx	#%00000011
	sir	$12
	rts
wc_ex2:
	sec
	bra	wc_ex1

time3:
	jsr	time
time2:
	jsr	time
time:
	rts

;**************************************************************************
;*
;* SECTION  jump table springboard
;*
;******
getbyte:
	jmp	(jumptable_getbyte)
putbyte:
	jmp	(jumptable_putbyte)
getmode:
	jmp	(jumptable_getmode)
putmode:
	jmp	(jumptable_putmode)
checkclk:
	jmp	(jumptable_checkclk)
restore:
	jmp	(jumptable_restore)
porttab:
	dc.w	tab_joy1,tab_joy2,tab_user0_4

tab_joy1:
	dc.w	getbyte_joy1
	dc.w	putbyte_joy1
	dc.w	getmode_joy1
	dc.w	putmode_joy1
	dc.w	checkclk_joy1
	dc.w	restore_joy1
tab_joy2:
	dc.w	getbyte_joy2
	dc.w	putbyte_joy2
	dc.w	getmode_joy2
	dc.w	putmode_joy2
	dc.w	checkclk_joy2
	dc.w	restore_joy2
tab_user0_4:
	dc.w	getbyte_user0_4
	dc.w	putbyte_user0_4
	dc.w	getmode_user0_4
	dc.w	putmode_user0_4
	dc.w	checkclk_user0_4
	dc.w	restore_user0_4

;**************************************************************************
;*
;* NAME  clear_progress, update_progress_inc
;*
;* DESCRIPTION
;*   handle progress indicator
;*   
;******
PROGRESS_POS	equ	9
PROGRESS_WIDTH	equ	7
ACTION_POS	equ	16
TOTAL_WIDTH	equ	8

clear_progress:
	lda	#0
	sta	current_progress
	ldx	#TOTAL_WIDTH
cpr_lp1:
	lda	#$20
	sta	$0400+PROGRESS_POS-1,x
	lda	#12
	sta	$d800+PROGRESS_POS-1,x
	dex
	bne	cpr_lp1
	rts
	
; Called with X remapped to reg $A (zp page). 
update_progress_inc:
; increase pointer MSB
	inc	ptr_zp+1

; handle progress indication
	inc	current_progress
	lda	current_progress
	lsr
	bcs	upi_ex1

	sir	$12
	tya
	pha

	jsr	update_progress

; restore state and exit
	pla
	tay	
	sir	$1a
upi_ex1:
	rts
		
prepare_progress:
	sta	$0400+ACTION_POS
update_progress:
	lda	current_progress
	lsr
	ldy	#0
upi_lp1:
	and	#$0f
	tax
	lda	prog_tab,x
	sta	$0400+PROGRESS_POS,y
	txa
	sec
	sbc	#4
	iny
	cpy	#PROGRESS_WIDTH
	bne	upi_lp1
	rts

prog_tab:
	dc.b	$f4,$e1,$6a,$20,$74,$61,$ea,$a0
	dc.b	$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0

;**************************************************************************
;*
;* NAME  check_speedcode, kill_speedcode
;*
;* DESCRIPTION
;*   speed code cache
;*   
;******
kill_speedcode:
	lda	#SPEED_NOTINSTALLED
check_speedcode:
	cmp	current_speedcode
	beq	csc_ex1
	sta	current_speedcode
csc_ex1:
	rts
	
;**************************************************************************
;*
;* NAME  get_header
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_READ_MEM ($01) or CMD_WRITE_MEM ($02)
;*	01: mode    (0=ram, 1=rom) (only used for CMD_READ_MEM)
;*	02: bank    ($00-$7f)
;*	03: ptr_lsb ($0000-$3fff)
;*	04: ptr_msb
;*	05: len_lsb ($0001-$4000)
;*	06: len_msb
;*   
;******
get_header:
; byte 00 (command) already gotten when here.

	jsr	getbyte		; 0=ram, 1=rom
	tax
	lda	#DEFAULT_REG8	; ram
	cpx	#1
	bne	gh_skp1
	and	#%01000101	; Segment 2 = flash
gh_skp1:
	sta	mode_zp
	jsr	getbyte
	sta	bank_zp
	jsr	getbyte
	sta	ptr_zp
	jsr	getbyte
	clc
	adc	#$80		; Segment 2
	sta	ptr_zp+1

	jsr	getbyte
	clc
	adc	ptr_zp
	php
	sta	end_zp
	jsr	getbyte
	plp
	adc	ptr_zp+1
	sta	end_zp+1
	rts
	
;**************************************************************************
;*
;* NAME  srv_cmd_read_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_READ_MEM ($01)
;*	01: mode    (0=ram, 1=rom) \
;*	02: bank    ($00-$7f)      |
;*	03: ptr_lsb ($0000-$3fff)  | (in get_header)
;*	04: ptr_msb                | 
;*	05: len_lsb ($0001-$4000)  |
;*	06: len_msb                /
;*   reply:
;*	00: databytes*len
;*	len: chksum 
;*
;*   the read access must be within the same 16Kb bank.
;*   
;******
srv_cmd_read_mem:
	lda	#"R"&$3f
	jsr	prepare_progress

	jsr	install_putblock
	
	jsr	get_header

	sir	$e8
	ldy	bank_zp
	ldx	mode_zp
	sir	$12

	jsr	putmode

	lda	#0
	sta	chk_zp	

;* transfer the data
	sir	$1a
	jsr	putblock
	sir	$12

;* send checksum
	lda	chk_zp	
	jsr	putbyte

	
	sir	$e8
	ldy	#$02
	ldx	#DEFAULT_REG8
	sir	$12

	jmp	getmode
; 	rts

;**************************************************************************
;*
;* NAME  install_putblock
;*
;* DESCRIPTION
;*   Install putblock speed code into stack area if it's not already
;*   there.
;*   
;******
install_putblock:
	ldy	#SPEED_PUTBLOCK
	ldx	#0
	lda	blockctrl_reg
	lsr
	bcc	ipb_skp1
	ldy	#SPEED_PUTBLOCK_SAFE
	ldx	#3
ipb_skp1:
	tya
	jsr	check_speedcode
	beq	ipb_ex1		; Already installed, skip!

	txa
	clc
	adc	store_port
	asl
	tax
	lda	pbltab,x
	sta	ptr_zp
	lda	pbltab+1,x
	sta	ptr_zp+1

	ldy	#0
ipb_lp1:
	lda	(ptr_zp),y
	sta	block_speedcode,y
	iny
	cpy	#PUTBLOCK_SAFE_LEN
	bne	ipb_lp1
ipb_ex1:
	rts

pbltab:
	dc.w	putblock_joy1,putblock_joy2,putblock_user0_4
	dc.w	putblock_safe_joy1,putblock_safe_joy2,putblock_safe_user0_4
	
putblock	equ	block_speedcode
		
;**************************************************************************
;*
;* NAME  srv_cmd_write_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_WRITE_MEM ($02)
;*	01: mode    (0=ram)        \
;*	02: bank    ($00-$7f)      |
;*	03: ptr_lsb ($0000-$3fff)  | (in get_header)
;*	04: ptr_msb                |
;*	05: len_lsb ($0001-$4000)  |
;*	06: len_msb                /
;*	07: databytes*len  
;*	07+len: chksum 
;*   reply:
;*	00:  chksum 
;*
;*   the write access must be within the same 16Kb bank.
;*   
;* RETURNS: Z=0 if ok, Z=1 if checksum error.
;*   
;******
srv_cmd_write_mem:
	lda	#"W"&$3f
	jsr	prepare_progress

	jsr	install_getblock

	jsr	get_header

	sac	$ee
	lda	bank_zp
	sac	$00

	lda	#0
	sta	chk_zp	

;* transfer the data
	sir	$1a
	jsr	getblock
	sir	$12

;* get and check the checksum
	jsr	getbyte
	cmp	chk_zp		; check for error
	php
	
;* send back our view of the checksum
	jsr	putmode
	lda	chk_zp	
	jsr	putbyte

	sac	$ee
	lda	#2
	sac	$00

	jsr	getmode
	plp
	rts

;**************************************************************************
;*
;* NAME  install_getblock
;*
;* DESCRIPTION
;*   Install getblock speed code into stack area if it's not already
;*   there.
;*   
;******
install_getblock:
	lda	#SPEED_GETBLOCK
	jsr	check_speedcode
	beq	igb_ex1		; Already installed, skip!

	lda	store_port
	asl
	tax
	lda	gbltab,x
	sta	ptr_zp
	lda	gbltab+1,x
	sta	ptr_zp+1

	ldy	#0
igb_lp1:
	lda	(ptr_zp),y
	sta	block_speedcode,y
	iny
	cpy	#GETBLOCK_LEN
	bne	igb_lp1
igb_ex1:
	rts

gbltab:
	dc.w	getblock_joy1,getblock_joy2,getblock_user0_4

getblock	equ	block_speedcode

;**************************************************************************
;*
;* NAME  srv_cmd_execute_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_EXECUTE_MEM ($03)
;*	01: ptr_lsb
;*	02: ptr_msb
;*
;******
srv_cmd_execute_mem:
	php	; SR 
	lda	#0
	pha	; Acc
	pha	; XR
	pha	; YR
	lda	#$07
	pha	; 01
	ldx	#1
	bra	scs_common
;**************************************************************************
;*
;* NAME  srv_cmd_sys
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_SYS ($04)
;*	01: mode (00=normal)
;*	02: ptr_lsb
;*	03: ptr_msb
;*	04: SR
;*	05: Acc
;*	06: XR
;*	07: YR
;*	08: 01
;*
;******
srv_cmd_sys:
	jsr	getbyte		; mode (not used)
	
	ldx	#0
scs_common:
	jsr	kill_speedcode

	ldy	#systramp_end-systramp-1
scs_lp1:
	lda	systramp,y
	sta	systramp_run,y
	dey
	bpl	scs_lp1

	jsr	getbyte
	sta	st_sm1+1
	jsr	getbyte
	sta	st_sm1+2

	txa
	bne	scs_skp1

	ldx	#5
scs_lp2:
	jsr	getbyte
	pha
	dex
	bne	scs_lp2

scs_skp1:
; clear up and run.
	jsr	uninit_scr_comm

	pla
	sta	$01
	pla
	tay
	pla
	tax
	pla
	
	sir	$d8
	sty	st_sm2+1
	stx	st_sm3+1
	jmp	systramp_run

; execute code
systramp:
	rorg	block_speedcode
systramp_run:
	ldy	#$01
	ldx	#%01010101
	sir	$12
	plp
st_sm1:
	jsr	st_sm1
	php
	sei
	sir	$d8
st_sm2:
	ldy	#$00
st_sm3:
	ldx	#$00
	jmp	scs_return
	rend
systramp_end:

; return to server
scs_return:
	sir	$12
	cld
	sta	store_acc
	stx	store_xr
	sty	store_yr
	pla
	sta	store_sr
	lda	#$07
	sta	$01
	
	jmp	init_scr_comm
; 	rts

;**************************************************************************
;*
;* NAME  srv_cmd_sys_result
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_SYS_RESULT ($05)
;*	01: mode (00=normal)
;*   reply:
;*	00: SR
;*	01: Acc
;*	02: XR
;*	03: YR
;*
;******
srv_cmd_sys_result:
	jsr	getbyte		; mode (not used)

	jsr	putmode
	lda	store_sr
	jsr	putbyte
	lda	store_acc
	jsr	putbyte
	lda	store_xr
	jsr	putbyte
	lda	store_yr
	jmp	putbyte
; 	rts

;**************************************************************************
;*
;* NAME  srv_cmd_init
;*
;* DESCRIPTION
;*   Return the implemented protocol revision.
;*
;*   command:
;*	00: command CMD_INIT ($40)
;*	01: mode (00 = normal basic, 01 = minimal basic, 
;*	          02 = normal kernal, 03 = minimal kernal)
;*
;******
srv_cmd_init:
	jsr	getbyte
	sta	mode_tmp	; Use mode_tmp instead of mode_zp because
				; inits may clear zp.
	
	lda	blockctrl_reg
	pha
	jsr	uninit_scr_comm

; reset init
	lda	mode_tmp
	lsr			; 01 or 03?
	bcc	sci_skp1	; no, skip to normal init.

	jsr	clone_fd50_lite
	bra	sci_skp2
sci_skp1:	
	jsr	$fda3
	jsr	clone_fd50
sci_skp2:
	jsr	clone_fd15
	jsr	$ff5b

; skip basic init for modes 02 and 03
	lda	mode_tmp
	lsr
	lsr		; 02 or 03?
	bcs	sci_ex1	; yes, skip basic init
		
; init basic vectors
	jsr	$e453

; preserve stack.
	tsx
	stx	sp_store
	ldx	#STACKCOPY_LEN-1
sci_lp1:
	lda	STACKCOPY_ADDR,x
	sta	stack_store,x
	dex
	bpl	sci_lp1

; init basic storage (sets $01fc and $01fd to $01)
	jsr	$e3bf
; new (resets stack pointer to $fa)
	jsr	$a644

; restore stack
	ldx	#STACKCOPY_LEN-1
sci_lp2:
	lda	stack_store,x
	sta	STACKCOPY_ADDR,x
	dex
	bpl	sci_lp2
	ldx	sp_store
	txs


sci_ex1:
	pla
	sta	blockctrl_reg
	jmp	init_scr_comm
;	rts

;**************************************************************************
;*
;* NAME  srv_cmd_load
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_LOAD ($41)
;*	01: mode (00=normal LOAD, 01=only CLR, no relink)
;*
;******
srv_cmd_load:
	jsr	kill_speedcode
	ldx	#loadtramp_end-loadtramp-1
ld_lp1:
	lda	loadtramp,x
	sta	loadtramp_run,x
	dex
	bpl	ld_lp1

	jsr	getbyte
	beq	ld_skp1
	lda	#$2c
	sta	lt_sm1
ld_skp1:
	
	lda	blockctrl_reg
	pha
	jsr	uninit_comm

; preserve stack.
	tsx
	stx	sp_store
	ldx	#STACKCOPY_LEN-1
ld_lp2:
	lda	STACKCOPY_ADDR,x
	sta	stack_store,x
	dex
	bpl	ld_lp2
	
	sir	$d8
	sty	lt_sm2+1
	stx	lt_sm3+1
	jmp	loadtramp_run

; execute code
loadtramp:
	rorg	block_speedcode
loadtramp_run:
	ldy	#$01
	ldx	#%01010101
	sir	$12
lt_sm1:
	jsr	$a533		; relink
	jsr	$a659		; clr
	sir	$d8
lt_sm2:
	ldy	#$00
lt_sm3:
	ldx	#$00
	jmp	ld_return
	rend
loadtramp_end:

ld_return:
	sir	$12

; restore stack
	ldx	#STACKCOPY_LEN-1
ld_lp3:
	lda	stack_store,x
	sta	STACKCOPY_ADDR,x
	dex
	bpl	ld_lp3
	ldx	sp_store
	txs
	
	jsr	init_comm
	pla
	sta	blockctrl_reg

 	rts
	
;**************************************************************************
;*
;* NAME  srv_cmd_run
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_RUN ($42)
;*	01: mode (00=normal RUN)
;*
;******
srv_cmd_run:
	ldx	#runtramp_end-runtramp-1
rn_lp1:
	lda	runtramp,x
	sta	runtramp_run,x
	dex
	bpl	rn_lp1

	jsr	getbyte

	jsr	uninit_scr_comm

; important for basic
	ldx	#$fb
	txs

; set string storage (normally done in $e3bf)
	lda	#$01
	sta	$01fc
	sta	$01fd

; turn of os messages for run
	lda	#$00
	jsr	$ff90
		
	sir	$d8
	jmp	runtramp_run

; execute code
runtramp:
	rorg	block_speedcode
runtramp_run:
	ldy	#$01
	ldx	#%01010101
	sir	$12
	cli
	jsr	$a659		; clr, setup chrget/chrgot ptr
	jmp	$a7ae		; run
	rend
runtramp_end:

;**************************************************************************
;*
;* NAME  srv_cmd_exit
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_EXIT ($43)
;*	01: mode (00=CLR and exit, 01=only exit)
;*
;******
srv_cmd_exit:
	ldx	#exittramp_end-exittramp-1
ex_lp1:
	lda	exittramp,x
	sta	exittramp_run,x
	dex
	bpl	ex_lp1

	jsr	getbyte
	beq	ex_skp1
	lda	#$2c
	sta	ext_sm1
ex_skp1:

	jsr	uninit_scr_comm

; important for basic
	ldx	#$fb
	txs

; set string storage (normally done in $e3bf)
	lda	#$01
	sta	$01fc
	sta	$01fd

	sir	$d8
	jmp	exittramp_run

; execute code
exittramp:
	rorg	block_speedcode
exittramp_run:
	ldy	#$01
	ldx	#%01010101
	sir	$12
	cli
ext_sm1:
	jsr	$a659
	jmp	$a47b		; exit to basic ($a474 without printing
				; READY.)
	rend
exittramp_end:

;**************************************************************************
;*
;* NAME  srv_cmd_print
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_PRINT ($60)
;*	01: len (00=$100)
;*      02: string*len
;*
;******
srv_cmd_print:
	jsr	uninit_scr_no_d020

	jsr	getbyte
	tax
scp_lp1:
	jsr	getbyte
	jsr	$e716
	sei
	dex
	bne	scp_lp1

;kill any characters gotten during the enabled
;interrupts. (not required because Timer A is stopped)
;	stx	$c6

	jmp	init_scr_no_d020
;	rts

;**************************************************************************
;*
;* NAME  srv_cmd_color
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_COLOR ($61)
;*	01: $D020
;*      02: $D021
;*	03: text color
;*
;******
srv_cmd_color:
	jsr	getbyte
	sta	store_d020
	jsr	getbyte
	sta	$d021
	jsr	getbyte
	sta	$0286
	rts

;**************************************************************************
;*
;* NAME  srv_cmd_query_revision
;*
;* DESCRIPTION
;*   Return the implemented protocol revision.
;*
;*   command:
;*	00: command CMD_QUERY_REVISION ($80)
;*   reply:
;*	00: major
;*	01: minor
;*
;******
srv_cmd_query_revision:
	jsr	putmode
	lda	#PREV_MAJOR
	jsr	putbyte
	lda	#PREV_MINOR
	jmp	putbyte
;	rts

;**************************************************************************
;*
;* NAME  srv_cmd_query_implementation
;*
;* DESCRIPTION
;*   Return the server implementation.
;*
;*   command:
;*	00: command CMD_QUERY_IMPLEMENTATION ($81)
;*   reply:
;*	00: len
;*	01: chars*len (not null terminated)
;*
;******
srv_cmd_query_implementation:
	jsr	putmode
	lda	#IMPL_LEN
	jsr	putbyte

	ldx	#0
impl_lp1:
	lda	impl_str,x
	jsr	putbyte
	inx
	cpx	#IMPL_LEN
	bne	impl_lp1
	rts
impl_str:
	dc.b	IMPLEMENTATION_STR
IMPL_LEN equ	.-impl_str


;**************************************************************************
;*
;* NAME  srv_cmd_query_config
;*
;* DESCRIPTION
;*   read server configuration.
;*   command:
;*	00: command CMD_QUERY_CONFIG ($82)
;*   reply:
;*	00: port ($00=joy1, $01=joy2, $02=user)
;*	01: mode (0=ram, 1=rom) \ 
;*	02: start_lsb           |
;*	03: start_midsb         | server location
;*	04: start_msb           | (typically the segment where the code
;*	05: end_lsb             | resides)
;*	06: end_midsb           |
;*	07: end_msb             /
;*
;******
srv_cmd_query_config:
	jsr	putmode
	lda	store_port
	jsr	putbyte

; ram or rom
	IFCONST	IN_ROM
	lda	#1
	ELSE
	lda	#0		; in ram
	ENDIF
	jsr	putbyte

; calculate start address from current segment
	lda	#0
	sta	ptr_zp
	sir	$1d
	txa
	sir	$12
	lsr
	ror	ptr_zp
	lsr
	ror	ptr_zp
	tax
; send start (ptr_zp=MidSB, X=MSB)
	lda	#0
	jsr	putbyte
	lda	ptr_zp
	jsr	putbyte
	txa
	jsr	putbyte
; send end
	lda	#TOTAL_SIZE & $ff
	jsr	putbyte
	lda	ptr_zp
	clc
	adc	#[TOTAL_SIZE >> 8] & $ff
	bcc	scrc_skp1
	inx
scrc_skp1:
	jsr	putbyte
	txa
	jmp	putbyte	
;	rts

;**************************************************************************
;*
;* NAME  srv_cmd_set_blockctrl
;*
;* DESCRIPTION
;*   Set the block control register.
;*   command:
;*	00: command CMD_SET_BLOCKCTRL ($83)
;*	01: value
;*
;******
srv_cmd_set_blockctrl:
	jsr	getbyte
	sta	blockctrl_reg
	rts

;**************************************************************************
;*
;* NAME  init_scr
;*
;* DESCRIPTION
;*   Initialize screen.
;*
;******
init_scr:
	lda	#$01
	sta	$d03f
	lda	$d020
	sta	store_d020

init_scr_no_d020:
	jsr	preserve_scr

	ldx	#40
in_lp1:
	lda	text-1,x
	and	#$3f
	sta	$0400-1,x
	lda	#1
	sta	$d800-1,x
	dex
	bne	in_lp1

; Acc*12
	lda	store_port
	asl
	adc	store_port
	asl
	asl
	tay
in_lp2:
	lda	str,y
	and	#$3f
	sta	$0400+18,x
	iny
	inx	
	cpx	#12
	bne	in_lp2

	jmp	clear_progress
;	rts

text:
	dc.b	"DTVTRANS                      , EXIT=R/S" 
str:
	dc.b	"$DC01 (JOY1)"
	dc.b	"$DC00 (JOY2)"
	dc.b	"$DD01 (USER)"

;**************************************************************************
;*
;* NAME  uninit_scr
;*
;* DESCRIPTION
;*   Restore screen to normal.
;*
;******
uninit_scr:
	lda	store_d020
	sta	$d020
uninit_scr_no_d020:
	jmp	restore_scr
;	rts

;**************************************************************************
;*
;* NAME  init_comm, init_scr_comm
;*
;* DESCRIPTION
;*   Initialize for communication.
;*
;******
init_scr_comm:
	jsr	init_scr
init_comm:
	sir	$e8
	sty	store_reg_e
	stx	store_reg_8
	sir	$12

; enable extended features
	lda	#$01
	sta	$d03f
; disable badlines
	lda	#%00100000
	sta	$d03c

; burst enable, skip internal cycle
	sac	$99
	lda	#%00000011
	sac	$00

	jsr	kill_speedcode

; reset block transfer options
	lda	#0
	sta	blockctrl_reg
	
; kill timerb on both CIA's to be able to use $dc07 and $dd07 as temporaries.
; 	lda	#%00000000	
	sta	$dc0f
	sta	$dd0f
; kill timer A IRQ on CIA #1 to avoid keyscan even when "CLI".
	lda	#$7f
	sta	$dc0d
	lda	$dc0d		; ack pending interrupts
	
	jmp	getmode
;	rts

;**************************************************************************
;*
;* NAME  uninit_comm, uninit_scr_comm
;*
;* DESCRIPTION
;*   Restore ports and banks.
;*
;******
uninit_scr_comm:
	jsr	uninit_scr
uninit_comm:	
; Default memory map.
	sir	$e8
	ldy	store_reg_e
	ldx	store_reg_8
	sir	$12

; all port bits as inputs, and wait for transmitter to tri-state
	jsr	restore

; reenable timer A IRQ on CIA #1
	lda	#$81
	sta	$dc0d

; disable burst, do not skip internal cycles anymore.
	sac	$99
	lda	#%00000000
	sac	$00

; enable badlines again
	lda	#%00000000
	sta	$d03c

; disable extended features
	lda	#$00
	sta	$d03f

	rts


;**************************************************************************
;*
;* NAME  misc port stuff
;*
;* DESCRIPTION
;*
;******
	MAC	checkclkmac
	lda	#%00001000
; Wait for CLK low
	bit	{1}
	ENDM

	MAC	getmodemac
; JoyA0-JoyA3=input, JoyA4=output
	IF	{1}=$dc00
	lda	#%00010000 | %11100000
	ELSE
	lda	#%00010000
	ENDIF
	sta	{1}+2
; all ones (tri-state)
	lda	#%11111111
	sta	{1}
	ENDM

	MAC	putmodemac
; JoyA3=input, JoyA0-JoyA2,JoyA4=output
	IF	{1}=$dc00
	lda	#%00010111 | %11100000
	ELSE
	lda	#%00010111
	ENDIF
	sta	{1}+2
; all ones (tri-state)
	lda	#%11111111
	sta	{1}
	ldx	#0
.pm_lp1:
	dex
	bne	.pm_lp1
	ENDM

	MAC	restoremac
; all port bits as inputs.
	lda	#%00000000
	sta	{1}+2
	lda	#%11111111
	sta	{1}
	IF	{1}!=$dd01
; wait for all port bits to go high before exiting
.uin_lp1:	
	lda	{1}
	and	#%00011111
	cmp	#%00011111
	bne	.uin_lp1
	ENDIF
	IF	{1}=$dc00
	lda	#%11111111
	sta	{1}+2
	ENDIF
	ENDM


;**************************************************************************
;*
;* NAME  getblockmac
;*
;* DESCRIPTION
;*   Get a block from the master.
;*   Called with X remapped to reg A (zp page).
;*   Note1: Must have constant length for all configurations!
;*   Note2: Must be position independant!
;*
;******
	MAC	getblockmac
.gbl_lp1:
	ldx	#{1} >> 8	; Map zp to CIA registers
	ldy	#%11101111
	lda	#%00001000
; Wait for CLK low
.gbl_lp2:
	bit	{1} & $ff
	bne	.gbl_lp2
; D2/D1/D0 contains databits 7/6/5
	lda	{1} & $ff
; ACK=0 by shifting CLK into ACK.
	asl
	asl
	sty	{1} & $ff
	asl
	nop
	sta	$07		; TimerB MSB ($dc07 or $dd07)
	lda	#%00001000
; Wait for CLK high
.gbl_lp3:
	bit	{1} & $ff
	beq	.gbl_lp3
; D2/D1/D0 contains databits 4/3/2
	lda	{1} & $ff
; ACK=1 by shifting CLK into ACK.
	ldy	#%11111111
	sty	{1} & $ff
	and	#%00000111
	ora	$07		; TimerB MSB ($dc07 or $dd07)
	asl
	asl
	sta	$07		; TimerB MSB ($dc07 or $dd07)
	lda	#%00001000
; Wait for CLK low
.gbl_lp4:
	bit	{1} & $ff
	bne	.gbl_lp4
; D1/D0 contains databits 1/0  D2 contains SYNC
	lda	{1} & $ff
; ACK=0 by shifting CLK into ACK.
	ldy	#%11101111
	sty	{1} & $ff
	and	#%00000011
	ora	$07		; TimerB MSB ($dc07 or $dd07)
	tay
	nop
	lda	#%00001000
; Wait for CLK high
.gbl_lp5:
	bit	{1} & $ff
	beq	.gbl_lp5
; ACK=1 by shifting CLK into ACK.
	lda	#%11111111
	sta	{1} & $ff
	sec
	tya
	ldx	#0		; Restore zp mapping to $0000

	ldy	#0
	sty	$01
	nop
	nop
	sta	(ptr_zp),y
	ldy	#$07
	sty	$01
; C=1 here
	adc	chk_zp
	sta	chk_zp
	nop
	nop
	inc	ptr_zp
	beq	.gbl_msb
.gbl_skp1:
	lda	ptr_zp
	cmp	end_zp
	bne	.gbl_lp1
	lda	ptr_zp+1
	cmp	end_zp+1
	bne	.gbl_lp1
	rts
	nop
.gbl_msb:
;	inc	ptr_zp+1
	jsr	update_progress_inc
_TMP_VAL	set	.gbl_skp1 ; workaround:	convert local var to global.
	bra	_TMP_VAL
	ENDM

;**************************************************************************
;*
;* NAME  putblockmac_safe
;*
;* DESCRIPTION
;*   Send a block to the master.  
;*   Called with X remapped to reg A (zp page).
;*   Note1: Must have constant length for all configurations!
;*   Note2: Must be position independant!
;*
;******
	MAC	putblockmac_safe
.pbl_lp1:
	ldy	#0
	sty	$01
	nop
	nop
	lda	(ptr_zp),y
	ldy	#$07
	sty	$01
	ldx	#{1} >> 8	; Map zp to CIA registers
	tay
	nop
	lda	#%00001000
; Wait for CLK low
.pbl_lp2:
	bit	{1} & $ff
	bne	.pbl_lp2
; databits 7/6/5 to D2/D1/D0, ACK=0
	tya
	lsr
	lsr
	lsr
	lsr
	lsr
	eor	#%00010000
	sta	{1} & $ff ; ACK=1
	eor	#%00010000
	sta	{1} & $ff ; ACK=0

	lda	#%00001000
; Wait for CLK high
.pbl_lp3:
	bit	{1} & $ff
	beq	.pbl_lp3

; databits 4/3/2 to D2/D1/D0, ACK=1
	nop
	tya
	lsr
	lsr
	and	#%00000111
	sta	{1} & $ff ; ACK=0
	eor	#%00010000
	sta	{1} & $ff ; ACK=1
	
	lda	#%00001000
; Wait for CLK low
.pbl_lp4:
	bit	{1} & $ff
	bne	.pbl_lp4

; databits 1/0 to D1/D0, SYNC to D2, ACK=0
	nop
	tya
	and	#%00000011
	eor	#%00010000
	sta	{1} & $ff ; ACK=1
	eor	#%00010000
	sta	{1} & $ff ; ACK=0

	lda	#%00001000
; Wait for CLK high
.pbl_lp5:
	bit	{1} & $ff
	beq	.pbl_lp5
; ACK=1 by shifting CLK into ACK.
	nop
	nop
	lda	#%11111111
	sta	{1} & $ff
	ldx	#0		; Restore zp mapping to $0000
	tya
	sec
	adc	chk_zp
	sta	chk_zp

	inc	ptr_zp
	beq	.pbl_msb
.pbl_skp1:
	lda	ptr_zp
	cmp	end_zp
	bne	.pbl_lp1
	lda	ptr_zp+1
	cmp	end_zp+1
	bne	.pbl_lp1
	rts
.pbl_msb
;	inc	ptr_zp+1
	jsr	update_progress_inc
_TMP_VAL	set	.pbl_skp1 ; workaround:	convert local var to global.
	bra	_TMP_VAL
	ENDM

;**************************************************************************
;*
;* NAME  putblockmac
;*
;* DESCRIPTION
;*   Send a block to the master.  
;*   Called with X remapped to reg A (zp page).
;*   Note1: Must have constant length for all configurations!
;*   Note2: Must be position independant!
;*
;******
	MAC	putblockmac
.pbl_lp1:
	ldy	#0
	sty	$01
	nop
	nop
	lda	(ptr_zp),y
	ldy	#$07
	sty	$01
	ldx	#{1} >> 8	; Map zp to CIA registers
	tay
	nop
	lda	#%00001000
; Wait for CLK low
.pbl_lp2:
	bit	{1} & $ff
	bne	.pbl_lp2
; databits 7/6/5 to D2/D1/D0, ACK=0
	tya
	lsr
	lsr
	lsr
	lsr
	lsr
	sta	{1} & $ff

	lda	#%00001000
; Wait for CLK high
.pbl_lp3:
	bit	{1} & $ff
	beq	.pbl_lp3

; databits 4/3/2 to D2/D1/D0, ACK=1
	nop
	tya
	lsr
	lsr
	and	#%00000111
	ora	#%00010000
	sta	{1} & $ff
	
	lda	#%00001000
; Wait for CLK low
.pbl_lp4:
	bit	{1} & $ff
	bne	.pbl_lp4

; databits 1/0 to D1/D0, SYNC to D2, ACK=0
	nop
	tya
	and	#%00000011
	sta	{1} & $ff

	lda	#%00001000
; Wait for CLK high
.pbl_lp5:
	bit	{1} & $ff
	beq	.pbl_lp5
; ACK=1 by shifting CLK into ACK.
	lda	#%11111111
	sta	{1} & $ff
	ldx	#0		; Restore zp mapping to $0000
	nop
	nop
	tya
	sec
	adc	chk_zp
	sta	chk_zp

	inc	ptr_zp
	beq	.pbl_msb
.pbl_skp1:
	lda	ptr_zp
	cmp	end_zp
	bne	.pbl_lp1
	lda	ptr_zp+1
	cmp	end_zp+1
	bne	.pbl_lp1
	rts
.pbl_msb
;	inc	ptr_zp+1
	jsr	update_progress_inc
_TMP_VAL	set	.pbl_skp1 ; workaround:	convert local var to global.
	bra	_TMP_VAL
	ENDM
	
;**************************************************************************
;*
;* NAME  getbytemac
;*
;* DESCRIPTION
;*   Get a byte from the master.  
;*     Returns value in Acc and updates flags Z and N.
;*     Preserves X.
;*
;******
	MAC	getbytemac
	sir	$1a
	ldx	#{1} >> 8
	lda	#%00001000
; Wait for CLK low
.gb_lp1:
	bit	{1} & $ff
	bne	.gb_lp1
; D2/D1/D0 contains databits 7/6/5
	lda	{1} & $ff
; ACK=0 by shifting CLK into ACK.
	asl	{1} & $ff
	asl
	asl
	asl
	sta	$07		; TimerB MSB ($dc07 or $dd07)
	lda	#%00001000
; Wait for CLK high
.gb_lp2:
	bit	{1} & $ff
	beq	.gb_lp2
; D2/D1/D0 contains databits 4/3/2
	lda	{1} & $ff
; ACK=1 by shifting CLK into ACK.
	asl	{1} & $ff
	and	#%00000111
	ora	$07		; TimerB MSB ($dc07 or $dd07)
	asl
	asl
	sta	$07		; TimerB MSB ($dc07 or $dd07)
	lda	#%00001000
; Wait for CLK low
.gb_lp3:
	bit	{1} & $ff
	bne	.gb_lp3
; D1/D0 contains databits 1/0  D2 contains SYNC
	lda	{1} & $ff
; ACK=0 by shifting CLK into ACK.
	asl	{1} & $ff
	and	#%00000011
	ora	$07		; TimerB MSB ($dc07 or $dd07)
	tay
	lda	#%00001000
; Wait for CLK high
.gb_lp4:
	bit	{1} & $ff
	beq	.gb_lp4
; ACK=1 by shifting CLK into ACK.
	asl	{1} & $ff
	ldx	#0
	sir	$12
	tya
	ENDM

;**************************************************************************
;*
;* NAME  putbytemac
;*
;* DESCRIPTION
;*   Send a byte to the master. 
;*   Acc=data,   preserved Acc and X
;*
;******
	MAC	putbytemac
	sir	$1a
	ldx	#{1}>>8
	tay
	lda	#%00001000
; Wait for CLK low
.pb_lp1:
	bit	{1} & $ff
	bne	.pb_lp1
; databits 7/6/5 to D2/D1/D0, ACK=0
	tya
	lsr
	lsr
	lsr
	lsr
	lsr
	eor	#%00010000
	sta	{1} & $ff  ; ACK=1
	eor	#%00010000
	sta	{1} & $ff  ; ACK=0

	lda	#%00001000
; Wait for CLK high
.pb_lp2:
	bit	{1} & $ff
	beq	.pb_lp2

; databits 4/3/2 to D2/D1/D0, ACK=1
	tya
	lsr
	lsr
	and	#%00000111
	sta	{1} & $ff ; ACK=0
	eor	#%00010000
	sta	{1} & $ff ; ACK=1
	
	lda	#%00001000
; Wait for CLK low
.pb_lp3:
	bit	{1} & $ff
	bne	.pb_lp3

; databits 1/0 to D1/D0, SYNC to D2, ACK=0
	tya
	and	#%00000011
	eor	#%00010000
	sta	{1} & $ff ; ACK=1
	eor	#%00010000
	sta	{1} & $ff ; ACK=0

	lda	#%00001000
; Wait for CLK high
.pb_lp4:
	bit	{1} & $ff
	beq	.pb_lp4
; ACK=1 by shifting CLK into ACK.
	asl	{1} & $ff
	tya
	ldx	#0
	sir	$12
	ENDM

;**************************************************************************
;*
;* SECTION  instantiation of port macros
;*
;******
	align	4
checkclk_joy1:
	checkclkmac	$dc01
	rts
getmode_joy1:
	getmodemac	$dc01
	rts
putmode_joy1:
	putmodemac	$dc01
	rts
getbyte_joy1:
	getbytemac	$dc01
getbyte_joy1_end:
	rts
putbyte_joy1:
	putbytemac	$dc01
putbyte_joy1_end:
	rts
getblock_joy1:
	getblockmac	$dc01
getblock_joy1_end:
putblock_joy1:
	putblockmac	$dc01
putblock_joy1_end:
putblock_safe_joy1:
	putblockmac_safe $dc01
putblock_safe_joy1_end:
restore_joy1:
	restoremac	$dc01
	rts
	align	4
checkclk_joy2:
	checkclkmac	$dc00
	rts
getmode_joy2:
	getmodemac	$dc00
	rts
putmode_joy2:
	putmodemac	$dc00
	rts
getbyte_joy2:
	getbytemac	$dc00
	rts
putbyte_joy2:
	putbytemac	$dc00
	rts
getblock_joy2:
	getblockmac	$dc00
putblock_joy2:
	putblockmac	$dc00
putblock_safe_joy2:
	putblockmac_safe $dc00
restore_joy2:
	restoremac	$dc00
	rts
	align	4
checkclk_user0_4:
	checkclkmac	$dd01
	rts
getmode_user0_4:
	getmodemac	$dd01
	rts
putmode_user0_4:
	putmodemac	$dd01
	rts
getbyte_user0_4:
	getbytemac	$dd01
	rts
putbyte_user0_4:
	putbytemac	$dd01
	rts
getblock_user0_4:
	getblockmac	$dd01
putblock_user0_4:
	putblockmac	$dd01
putblock_safe_user0_4:
	putblockmac_safe $dd01
restore_user0_4:
	restoremac	$dd01
	rts

GETBYTE_LEN	equ	getbyte_joy1_end-getbyte_joy1
	echo	"[dtvtrans] getbyte len:",GETBYTE_LEN
PUTBYTE_LEN	equ	putbyte_joy1_end-putbyte_joy1
	echo	"[dtvtrans] putbyte len:",PUTBYTE_LEN

GETBLOCK_LEN	equ	getblock_joy1_end-getblock_joy1
	echo	"[dtvtrans] getblock len:",GETBLOCK_LEN
PUTBLOCK_LEN	equ	putblock_joy1_end-putblock_joy1
	echo	"[dtvtrans] putblock len:",PUTBLOCK_LEN
PUTBLOCK_SAFE_LEN equ	putblock_safe_joy1_end-putblock_safe_joy1
	echo	"[dtvtrans] putblock_safe len:",PUTBLOCK_SAFE_LEN

LARGEST_SPEEDCODE	set	PUTBLOCK_LEN
	if	GETBLOCK_LEN > LARGEST_SPEEDCODE
LARGEST_SPEEDCODE	set	GETBLOCK_LEN
	endif
	if	PUTBLOCK_SAFE_LEN > LARGEST_SPEEDCODE
LARGEST_SPEEDCODE	set	PUTBLOCK_SAFE_LEN
	endif
	echo	"[dtvtrans] largest speedcode:",LARGEST_SPEEDCODE
; eof
