/*************************************************************************
 *
 * FILE  timer.hpp
 * Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
 * Copyright (c) 2006, 2007 Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 * Written by Daniel Kahlin <daniel@kahlin.net> &
 *            Hannu Nuotio <hannu.nuotio@pp.inet.fi>
 * $Id: timer.hpp,v 1.10 2008-01-04 15:51:44 tlr Exp $
 *
 * DESCRIPTION
 *   Template for C++ header files.
 *
 ******/
#ifndef TIMER_HPP
#define TIMER_HPP

#include "../util/types.hpp"
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <unistd.h>

/*************************************************************************
 *
 * CLASS  Timer
 *
 * DESCRIPTION
 *
 * KNOWN BUGS
 *   none
 *
 ******/
class Timer
{
public:
    Timer() {}
    void start() { gettimeofday(&start_, NULL); }    
    void stop() { gettimeofday(&stop_, NULL); }    
    uint32_t get_time() const { return (stop_.tv_sec - start_.tv_sec)*1000+(stop_.tv_usec - start_.tv_usec)/1000; }
    double get_time_d() const { return (double)get_time()/1000; }

private:
    timeval start_;
    timeval stop_;
};

class SSleep
{
public:
    SSleep() { }

    void sleep(int t)
    {
	// just call unistd.h: sleep()
	::sleep(t);
    }

    void msleep(int t)
    {
	// call unistd.h: sleep() for seconds
	if (t >= 1000) {
	    ::sleep(t/1000);
	    t %= 1000;
	}
	// and usleep() for the rest
	::usleep(t*1000);
    }
};

class USleep
{
public:
    USleep() {}

    static void usleep(int t)
    {
    timeval ts;
    uint32_t stop, cur, start;
    
    gettimeofday(&ts, NULL);
    cur = ts.tv_sec*1000000+ts.tv_usec;
    start = cur;
    stop = cur + t;

    while ((cur<stop) || ((stop<start) && (cur>=start))) {
       gettimeofday(&ts, NULL);
       cur = ts.tv_sec*1000000+ts.tv_usec;
       }
    }
};


#endif /* TIMER_HPP */

/* eof */
