/*************************************************************************
 *
 * FILE  parport_transfer.hpp
 * Copyright (c) 2006, 2007, 2008 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: parport_transfer.hpp,v 1.30 2008-02-17 22:45:36 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <string>
#include "util/types.hpp"
#include "transfer.hpp"
// arch dependant
#include "port.hpp"

class ParportTransfer : public Transfer
{
public:
    // memory type commands
    static const int CMD_READ_MEM       = 0x01;
    static const int CMD_WRITE_MEM      = 0x02;
    static const int CMD_EXECUTE_MEM    = 0x03;
    static const int CMD_SYS            = 0x04;
    static const int CMD_SYS_RESULT     = 0x05;

    // basic type commands
    static const int CMD_INIT           = 0x40;
    static const int CMD_LOAD           = 0x41;
    static const int CMD_RUN            = 0x42;
    static const int CMD_EXIT           = 0x43;

    // screen type commands
    static const int CMD_PRINT          = 0x60;
    static const int CMD_COLOR          = 0x61;

    // configuration type commands
    static const int CMD_QUERY_REVISION = 0x80;
    static const int CMD_QUERY_IMPLEMENTATION = 0x81;
    static const int CMD_QUERY_CONFIG   = 0x82;
    static const int CMD_SET_BLOCKCTRL  = 0x83;

    // special commands
    static const int CMD_NOP            = 0xff;


    ParportTransfer(const std::string& portbase, int blocksize, bool use_safe_setup_dtv);
    virtual ~ParportTransfer() { }
    virtual void execute_mem(uint16_t addr);
    virtual void sys(uint16_t addr, uint8_t acc, uint8_t xr, uint8_t yr, uint8_t sr, uint8_t mem);
    virtual void sys_result(uint8_t& acc, uint8_t& xr, uint8_t& yr, uint8_t& sr);

    virtual void init(init_mode_t mode);
    virtual void load(load_mode_t mode);
    virtual void run(run_mode_t mode);
    virtual void exit(exit_mode_t mode);
    virtual void color(uint8_t border, uint8_t background, uint8_t text);

    virtual bool check_present(int timeout);
    virtual void reset(bool knock);
    virtual void tristate();

    virtual const char* get_implementation();

    virtual void send_file_boot(uint16_t start, uint16_t end, uint8_t* data);

protected:
    virtual void read_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* target);
    virtual void write_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* source);
    virtual void print_raw(const char* str, int len);
    virtual void get_revision_raw();
    virtual void get_configuration_raw();
    virtual void set_timing_raw(bool safe_setup_dtv);

private:
    void release_();
    void send_byte_(uint8_t data);
    uint8_t recv_byte_();
    void send_byte_boot_(uint8_t data);

    Port pt_;
};

/* eof */
