/*************************************************************************
 *
 * FILE  transfer.hpp
 * Copyright (c) 2006, 2007, 2008 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: transfer.hpp,v 1.28 2008-02-17 22:45:36 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/
#ifndef TRANSFER_HPP
#define TRANSFER_HPP

#include <string>
#include "util/types.hpp"
#include "membuffer.hpp"

class Transfer
{
public:
    Transfer(int blocksize, bool use_safe_setup_dtv);
    virtual ~Transfer() { }

    virtual void read_mem(uint8_t mode, uint32_t base, uint32_t len, uint8_t* target);
    virtual void read_mem(MemBuffer& mb);
    virtual uint16_t read_word(uint32_t addr);
    virtual uint8_t read_byte(uint32_t addr);

    virtual void write_mem(uint8_t mode, uint32_t base, uint32_t len, uint8_t* source);
    virtual void write_mem(const MemBuffer& mb);
    virtual void write_word(uint32_t addr, uint16_t data);
    virtual void write_byte(uint32_t addr, uint8_t data);

    virtual void execute_mem(uint16_t addr);
    virtual void sys(uint16_t addr, uint8_t acc, uint8_t xr, uint8_t yr, uint8_t sr, uint8_t mem);
    virtual void sys_result(uint8_t& acc, uint8_t& xr, uint8_t& yr, uint8_t& sr);

    enum init_mode_t {
	INIT_FULL_BASIC     = 0x00,
	INIT_MINIMAL_BASIC  = 0x01,
	INIT_FULL_KERNAL    = 0x02,
	INIT_MINIMAL_KERNAL = 0x03
    };
    virtual void init(init_mode_t mode);

    enum load_mode_t {
	LOAD_NORMAL         = 0x00,
	LOAD_NO_RELINK      = 0x01
    };
    virtual void load(load_mode_t mode);

    enum run_mode_t {
	RUN_NORMAL          = 0x00
    };
    virtual void run(run_mode_t mode);

    enum exit_mode_t {
	EXIT_NORMAL         = 0x00,
	EXIT_ONLY_EXIT      = 0x01
    };
    virtual void exit(exit_mode_t mode);

    virtual void print(const char* str);
    virtual void color(uint8_t border, uint8_t background, uint8_t text);

    virtual bool check_present(int timeout);
    virtual void reset(bool knock);
    virtual void tristate();

    virtual void send_file_boot(uint16_t start, uint16_t end, uint8_t* data);

    int get_rev_major() { get_revision_(); return prev_major_; };
    int get_rev_minor() { get_revision_(); return prev_minor_; };
    int get_conf_port() { get_configuration_(); return conf_port_; };
    bool get_conf_rom() { get_configuration_(); return conf_rom_; };
    uint32_t get_conf_start() { get_configuration_(); return conf_sa_; };
    uint32_t get_conf_end() { get_configuration_(); return conf_ea_; };
    virtual const char* get_implementation();

protected:
    virtual void read_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* target);
    virtual void write_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* source);
    virtual void print_raw(const char* str, int len);
    virtual void get_revision_raw();
    virtual void get_configuration_raw();
    virtual void set_timing_raw(bool safe_setup_dtv);

    virtual void get_revision_();
    virtual void get_configuration_();
    virtual void set_timing_(bool safe_setup_dtv);
    virtual void clear_state_();

    int blocksize_;
    bool use_safe_setup_dtv_;

    bool safe_timing_pc_;
    bool safe_setup_dtv_;

    bool got_revision_;
    // revision
    int prev_major_;
    int prev_minor_;

    // implementation string
    char impl_str_[256];

    bool got_configuration_;
    // configuration
    int conf_port_;
    bool conf_rom_;
    uint32_t conf_sa_;
    uint32_t conf_ea_;

};

#endif /* TRANSFER_HPP */
/* eof */
