/*************************************************************************
 *
 * FILE  getopt.hpp
 * Copyright (c) 2005 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: getopt.hpp,v 1.5 2007-12-26 22:44:02 tlr Exp $
 *
 * DESCRIPTION
 *   Template for C++ header files.
 *
 ******/
#ifndef GETOPT_HPP
#define GETOPT_HPP

#include <map>

/*************************************************************************
 *
 * CLASS  Getopt
 *
 * DESCRIPTION
 *   original <unistd.h> prototypes:
 *     int getopt(int argc, char * const argv[], const char *optstring);
 *     extern char *optarg;
 *     extern int optind, opterr, optopt;
 *
 * KNOWN BUGS
 *   none
 *
 ******/
class Getopt
{
public:
    class Opt
    {
    public:
	enum Status { OK, INVALID, MISSING };

	Opt()
	    : end_(false), status_(OK)
	{
	}
	Opt(bool end, enum Status status, char opt=0)
	    : end_(end), status_(status), opt_(opt)
	{
	}
	Opt(char opt, const char* arg) 
	    : end_(false), status_(OK), opt_(opt), arg_(arg)
	{
	}
	bool is_end() const { return end_; }
	int get_status() const { return status_; }
	char get_opt() const { return opt_; }
	const char* get_arg() const { return arg_; }

    private:
	bool end_;
	enum Status status_;
	char opt_;
        const char* arg_;
    };

    Getopt(int& argc, char **&argv, const char* optstring);
    ~Getopt();
    void set_optstring(const char* optstring);
    void set_endmarker(char c);
    bool skip_endmarker();
    Opt next_opt();
    const char* next_arg();
    int args_left() const { return cur_argc_-optind_; }

private:
    int& argc_;
    char**& argv_;
    int optind_;
    std::map<char,bool> optmap_;
    int cur_argc_;
    char endmarker_;
};

#endif /* GETOPT_HPP */

/* eof */
